@@ -32,7 +32,6 @@ def run_exists(query):
 
 def run_select(query):
     result = []
-
     try:
         db = mysql.connect(host=BD_HOST,
                 database=BD_DATABASE,
@@ -138,6 +137,45 @@ def get_n_objects_by_key(objectName, n=1, key='id', keyValue=1):
         return elements[:n]
     return None
 
+def update_object_by_conditions(objectName, fields, conditions):
+
+    parsed_fields = ""
+    parsed_conditions = ""
+    for key in fields:
+        value = fields[key]
+        if isinstance(value, str):
+            value = '"' + value + '"'
+        parsed_fields += f'{key} = {value}, '
+    else:
+        parsed_fields = parsed_fields[:-2]
+
+    for key in conditions:
+        operator = '='
+        value = conditions[key]
+        if isinstance(value, list):
+            operator = 'IN'
+            value = '(' + str(value)[1:-1] + ')'
+        elif isinstance(value, str):
+            value = '"' + value + '"'
+        parsed_conditions += f'{key} {operator} {value} AND '
+    else:
+        parsed_conditions = parsed_conditions[:-4]
+    query = f'UPDATE neuralplatform_{objectName.lower()} SET {parsed_fields} WHERE {parsed_conditions};'
+    run_update(query)
+
+def get_pagesInfo_of_request(request_id):
+    pagesInfo = {"pagesInfo": []}
+    request   = get_n_objects_by_key('request', 1, 'id', request_id)[0]
+    pages     = get_n_objects_by_key('page', None, 'document_id', int(request['document_id'])) or []
+    for page in pages:
+        class_name = ""
+        if page['tagged']:
+            classification = get_n_objects_by_key('classification', 1, 'page_id', int(page['id']))[0]
+            class_id = int(classification['classDefinition_id'])
+            class_name = get_n_objects_by_key('classDefinition', 1, 'id', class_id)[0]['name']
+        pagesInfo['pagesInfo'].append({"uri": page['imgUri'], "class": class_name})
+    return pagesInfo
+
 def update_object_by_key(objectName, key='id', keyValue=1, fields=None):
 
     if fields:
@@ -163,6 +201,18 @@ def get_projects_of_projectManager(projectManager_id):
 	project_ids = run_select(query)
 	return project_ids
 
+def get_bucketName_by_page_id(page_id):
+    try:
+        query = "SELECT s3Bucket FROM neuralplatform_account WHERE id = " + \
+            "(SELECT account_id FROM neuralplatform_project WHERE id = " + \
+            "(SELECT project_id FROM neuralplatform_dataset WHERE id = " + \
+            "(SELECT dataset_id FROM neuralplatform_document WHERE id = " + \
+            f"(SELECT document_id FROM neuralplatform_page WHERE id = {page_id}))));"
+        return run_select(query)[0]['s3Bucket']
+    except Exception as e:
+        print(f"get_bucketName_by_page_id : Error : {e}")
+        return ""
+
 def validate_projectManager(account_code, username, hashed_password):
     accounts = get_n_objects_by_key('account', 1, 'code', account_code)
     if accounts:
@@ -191,9 +241,10 @@ def insert_document(uploadDate, filename, extension, phase, uri, nPages, tagged,
 	return run_insert(query)
 
 def insert_request(phase, requestDate, document_id, project_id):
-	query = f'INSERT INTO neuralplatform_request(phase, requestDate, document_id, project_id) ' + \
-			f'VALUES ("{phase}", "{requestDate}", {document_id}, {project_id});'
-	return run_insert(query)
+    emptyJson = '"{}"'
+    query = f'INSERT INTO neuralplatform_request(phase, requestDate, document_id, project_id, response, status) ' + \
+            f'VALUES ("{phase}", "{requestDate}", {document_id}, {project_id}, {emptyJson}, "pending");'
+    return run_insert(query)
 
 def insert_page(imgUri, ocrUri, document_id):
     query = f'INSERT INTO neuralplatform_page(imgUri, ocrUri, document_id) ' + \
@@ -217,6 +268,11 @@ def get_pending_and_unblocked_steps():
         unblocked_steps = list(filter(lambda x: x['stepDefinition_id'] in unblocked_stepDefinitions, pending_steps))
     return unblocked_steps
 
+def insert_manualStep(startTime, result, request_id):
+    query = f'INSERT INTO neuralplatform_manualstep(startTime, beingTagged, completed, result, request_id) ' + \
+            f'VALUES ("{startTime}", 0, 0, "{result}", {request_id});'
+    return run_insert(query)
+
 ######## DEPRECATED ########
 
 def get_user_pk_by_username(username):
